<?php

/**
 * This software package is licensed under AGPL, Commercial license.
 *
 * @package maslosoft/addendum
 * @licence AGPL, Commercial
 * @copyright Copyright (c) Piotr Masełkowski <pmaselkowski@gmail.com> (Meta container, further improvements, bugfixes)
 * @copyright Copyright (c) Maslosoft (Meta container, further improvements, bugfixes)
 * @copyright Copyright (c) Jan Suchal (Original version, builder, parser)
 * @link http://maslosoft.com/addendum/ - maslosoft addendum
 * @link https://code.google.com/p/addendum/ - original addendum project
 */

namespace Maslosoft\Addendum\Collections;

use Maslosoft\Addendum\Interfaces\AnnotationEntityInterface;
use ReflectionProperty;

/**
 * Container for metadata generated by property annotations
 *
 * @author Piotr
 */
class MetaProperty implements AnnotationEntityInterface
{

	use \Maslosoft\Addendum\Traits\MetaState;

// <editor-fold defaultstate="collapsed" desc="Access Control">
	/**
	 * Indicates if field has getter
	 * @var bool
	 */
	public $callGet = false;

	/**
	 * Indicates if field has setter
	 * @var bool
	 */
	public $callSet = false;

	/**
	 * Indicates if field has either getter or setter
	 * @var bool
	 */
	public $direct = false;

	/**
	 * Getter method name
	 * @var string
	 */
	public $methodGet = '';

	/**
	 * Setter method name
	 * @var string
	 */
	public $methodSet = '';

	/**
	 * True if property is static
	 * @var bool
	 */
	public $isStatic = false;
// </editor-fold>
// <editor-fold defaultstate="collapsed" desc="Default value and property name">
	/**
	 * Default value of field as defined in class declaration
	 * @var mixed
	 */
	public $default = null;

	/**
	 * Name of a field
	 * @var string
	 */
	public $name = '';

// </editor-fold>

	/**
	 * Class constructor, sets some basic data for field
	 * @param ReflectionProperty $info
	 */
	public function __construct(ReflectionProperty $info = null)
	{
		// For internal use
		if (null === $info)
		{
			return;
		}
		$this->name = $info->name;
		$this->methodGet = 'get' . ucfirst($this->name);
		$this->methodSet = 'set' . ucfirst($this->name);
		$this->isStatic = $info->isStatic();
	}

	public function __get($name)
	{
		return null;
	}

}
